/*
 * Debug program interface IOCTL handler
 * for NetWinder by Corel Computer
 * Woody 1998 woody@corelcomputer.com
 */
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/config.h>

#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <linux/proc_fs.h>
#include <asm/segment.h>
//#include <unistd.h>
#include <linux/sched.h>
#include <linux/miscdevice.h>

#define VERSION$ "0.10"
/*****************************************************************************/
#include "nwdebug.h"

//void nwdebchar(char c);
//void nwdebstr(char *p);
//int nwprintk(const char *fmt, ...);

static int	open_nwdebug(struct inode* inodep, struct file* filep);
static void	release_nwdebug(struct inode* inodep, struct file* filep);
static int	nwdebug_ioctl(struct inode* inodep, struct file* filep, unsigned int cmd, unsigned long arg);
//static void	delay10ms(void);
//static void	delay1us(void);

static const char nwdebug_name[] = "nwdebug";

static struct file_operations nwdebug_fops = {
	NULL,			/* lseek              */
	NULL,			/* read               */
	NULL,			/* write              */
	NULL,			/* no special readdir            */
	NULL,			/* no special select             */
	nwdebug_ioctl,
	NULL,			/* no special mmap               */
	open_nwdebug,
	release_nwdebug,
	NULL,			/* no special fsync              */
	NULL,			/* no special fasync             */
	NULL,			/* no special check_media_change */
	NULL			/* no special revaldate          */
};

#if 0
//pull down reset line, while waiting...
void    delay_wait(int ticks)
{
    //wait ticks*10 ms
    current->state = TASK_INTERRUPTIBLE;
    current->timeout = jiffies + ticks;
    schedule();
}

void	delay10ms(void)
{
    //wait good 10 ms
    delay_wait(2);
}

void	delay1us(void)
{
    //wait 1 us
    inb(0x80);
}
#endif	

#if 0
void nwdebchar(char c)
{
volatile char* p = (volatile char *)0xE00003F8;
int timeout = 0x100000;

	while(!(p[5]&0x40) && timeout)
		{
		timeout--;
		}
	p[0] = c;
}

void nwdebstr(char *p)
{
char c;

	while ((c=*p)!='\0')
	{
		if (c == '\n')
		{
			nwdebchar('\r');
		}
		nwdebchar(c);
		p++;
	}
}
#endif

int nwprintk(const char *fmt, ...)
{
va_list args;
int i;
char *buf_end;
char buf[256];
long flags;

	save_flags(flags);
	cli();
	va_start(args, fmt);
	i = vsprintf(buf, fmt, args); /* hopefully i < sizeof(buf)-1 */
	buf_end = buf + i;
	va_end(args);
	restore_flags(flags);

//	nwdebstr(buf);
	putstring(buf);

	return(i);
}


static int open_nwdebug(struct inode* inodep, struct file* filep)
{
	MOD_INC_USE_COUNT;

	return 0;
}


static void release_nwdebug(struct inode* inodep, struct file* filep)
{
	MOD_DEC_USE_COUNT;
}


static int nwdebug_ioctl(struct inode* inodep, struct file* filep, unsigned int cmd, unsigned long arg)
{
int	i;
int	bits,mask,oldCPLD;

	printk(KERN_DEBUG "nwdebug_ioctl: cmd = 0x%X, arg = 0x%X.\n",cmd, *(int*)arg);

	switch (cmd)
	{
		case CMD_WRITE_CPLD:
			//expect 1 integer - bits, we need to create mask=0x0E
			//to prevent touching flash_writeable bit (bit 0)
			memcpy_fromfs(&i, (void*)arg, sizeof(i));
			oldCPLD = get_CPLD();	//get current setting
			mask = 0x7FFFFFFE;	//w/o top bit to be affected
			bits = i & 0x7FFFFFFE;	//7 bits to be set
			set_CPLD(bits, mask);	
//			printk(KERN_DEBUG "nwdebug: Old CPLD 0x%X, new CPLD 0x%X.\n",
//			    oldCPLD, get_CPLD());
			break;
		    

		case CMD_READ_CPLD:
			//return 1 integer
			i=get_CPLD();	//the HIGH bit is already stripped
//			printk(KERN_DEBUG "nwdebug: Current CPLD 0x%X.\n",i);
			memcpy_tofs((void*)arg, &i, sizeof(i));
			break;
		    

		case CMD_GET_LED:
			//return 1 integer
			i=getLED();
			memcpy_tofs((void*)arg, &i, sizeof(i));
			break;
		    

		case CMD_SET_LED:
			//expect 1 integer - LED state
			// (0=off, 1=green, 2=red, 3=yellow)
			memcpy_fromfs(&i, (void*)arg, sizeof(i));
			setLED(i,(int)&nwdebug_ioctl);
			break;
		    


		default:
			return -EINVAL;
	}

	return 0;
}


static struct miscdevice nwdebug_miscdev=
{
	NW_DEBUG_MINOR,
	"nwdebug",
	&nwdebug_fops
};

void nwdebug_init(void)
{
    misc_register(&nwdebug_miscdev);
    printk(KERN_NOTICE "Netwinder debug driver version %s.\n", VERSION$);

}

#ifdef MODULE
int init_module(void)
{
	nwdebug_init();
	return 0;
}

void cleanup_module(void)
{
	misc_deregister(&nwdebug_miscdev);
}
#endif

