//#define PCI_DEBUG 1
/*
 * PCI support
 * -- rough emulation of "PCI BIOS" functions
 *
 * Based on arch/ppc/kernel/ppc.c
 *
 * This version is NetWinder specific.
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/bios32.h>
#include <linux/pci.h>
#include <linux/kernel.h>
#include <asm/hardware.h>

/*
 * PCI interrupt configuration.
 */
 
 
/* Which PCI interrupt line does a given device [slot] use? */
/* Note: This really should be two dimensional based in slot/pin used */
unsigned char *Motherboard_map;

/* How is the 82378 PIRQ mapping setup? */
unsigned char *Motherboard_routes;

/* Tables for known hardware */   

/* Corel NetWinder */
static char NetWinder_pci_IRQ_map[5] =
  {
  	0,	/* Slot 0  - vga */
  	0,	/* Slot 1  - daughter card (ether100/scsi/etc) */
  	0,	/* Slot 2  - daughter card  */
  	0,	/* Slot 3  - ide */
  	0	/* Slot 4  - ether10 */
  };

static char Blackhawk_pci_IRQ_routes[] =
   {
   	0,	/* Line 0 - Unused */
   	9,	/* Line 1 */
   	11,	/* Line 2 */
   	14,	/* Line 3 */
   	15	/* Line 4 */
   };
   

/* #define PCI_DEBUG */

#ifdef PCI_STATS
int PCI_conversions[2];
#endif

unsigned long pcibios_init(unsigned long mem_start,
			   unsigned long mem_end)
{
	return mem_start;
}

unsigned long pcibios_fixup(unsigned long mem_start, unsigned long mem_end)
{
  return mem_start;
}



int
pcibios_present (void)
{
#ifdef PCI_DEBUG	
	printk("PCI [BIOS] present?\n");
#endif	
	return (1);
}

#define MK_PCI_ADDRESS(d,ofs) (IO_BASE_PCI_CONFIG \
    + (0x00080000 << PCI_SLOT(d)) \
    | (0x100 * PCI_FUNC(d)) \
    | ofs)

int
pcibios_read_config_dword (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned int *val)
{
	unsigned long _val;
	unsigned long *ptr;
#ifdef PCI_DEBUG	
        printk("PCI Read config dword[%d.%d.%d.%x] = ",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset);
#endif	
	if ((bus != 0) || (dev >= 40))
	{
		*val = 0xFFFFFFFF;
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned long *)MK_PCI_ADDRESS(dev,offset);
#ifdef PCI_DEBUG	
		printk("[%x] ", ptr);
#endif		
		_val = *ptr;
	}
#ifdef PCI_DEBUG	
	printk("%x\n", _val);
#endif	
	*val = _val;
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_read_config_word (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned short *val)
{
	unsigned short _val;
	unsigned short *ptr;
#ifdef PCI_DEBUG	
        printk("PCI Read config word[%d.%d.%d.%x] = ",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset);
#endif	
	if ((bus != 0) || (dev >= 40))
	{
		*val = 0xFFFFFFFF;
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned short *)MK_PCI_ADDRESS(dev,offset);
#ifdef PCI_DEBUG	
		printk("[%x] ", ptr);
#endif		
		_val = *ptr;
	}
#ifdef PCI_DEBUG	
	printk("%x\n", _val);
#endif		
	*val = _val;
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_read_config_byte (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned char *val)
{
	unsigned char _val;
	volatile unsigned char *ptr;
        
#ifdef PCI_DEBUG	
        printk("PCI Read config byte[%d.%d.%d.%x] = ",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset);
#endif		
	if ((bus != 0) || (dev >= 40))
	{
		*val = 0xFFFFFFFF;
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned char *)MK_PCI_ADDRESS(dev,offset);
#ifdef PCI_DEBUG	
		printk("[%x] ", ptr);
#endif		
		_val = *ptr;
	}
#ifdef PCI_DEBUG	
	printk("%x\n", _val);
#endif
	*val = _val;
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_write_config_dword (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned int val)
{
	unsigned long *ptr;
#ifdef PCI_DEBUG	
        printk("PCI Write config dword[%d.%d.%d.%x] = %x\n",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset, val);
#endif		
	if ((bus != 0) || (dev >= 40))
	{
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned long *)MK_PCI_ADDRESS(dev,offset);
            *ptr = val;
	}
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_write_config_word (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned short val)
{
	unsigned short *ptr;
#ifdef PCI_DEBUG	
        printk("PCI Write config word[%d.%d.%d.%x] = %x\n",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset, val);
#endif		
	if ((bus != 0) || (dev >= 40))
	{
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned short *)MK_PCI_ADDRESS(dev,offset);
            *ptr = val;
	}
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_write_config_byte (unsigned char bus,
    unsigned char dev, unsigned char offset, unsigned char val)
{
	unsigned char *ptr;
#ifdef PCI_DEBUG	
        printk("PCI Write config byte[%d.%d.%d.%x] = %x\n",
               bus, PCI_SLOT(dev), PCI_FUNC(dev), offset, val);
#endif		
	if ((bus != 0) || (dev >= 40))
	{
		return PCIBIOS_DEVICE_NOT_FOUND;
	} else
	{
            ptr = (unsigned char *)MK_PCI_ADDRESS(dev,offset);
            *ptr = val;
	}
	return PCIBIOS_SUCCESSFUL;
}

int
pcibios_find_device (unsigned short vendor, unsigned short device_id,
		     unsigned short index, unsigned char *bus,
		     unsigned char *dev)
{
	unsigned long w, desired = (device_id << 16) | vendor;
	int devnr;

	if (vendor == 0xffff) {
		return PCIBIOS_BAD_VENDOR_ID;
	}

	for (devnr = 0;  devnr < 5;  devnr++)
	{
		pcibios_read_config_dword(0, devnr<<3, PCI_VENDOR_ID, &w);
		if (w == desired) {
			if (index == 0) {
				*bus = 0;
				*dev = devnr<<3;
				return PCIBIOS_SUCCESSFUL;
			}
			--index;
		}
	}
	return PCIBIOS_DEVICE_NOT_FOUND;
}

int
pcibios_find_class (unsigned int class_code, unsigned short index, 
    unsigned char *bus, unsigned char *dev)
{
	int dev_nr, class, indx;
	indx = 0;
#ifdef PCI_DEBUG	
	printk("pcibios_find_class - class: %x, index: %x", class_code, index);
#endif	
	for (dev_nr = 0;  dev_nr < 40;  dev_nr++)
	{
		pcibios_read_config_dword(0, dev_nr, PCI_CLASS_REVISION, &class);
		if ((class>>8) == class_code)
		{
			if (index == indx)
			{
				*bus = 0;
				*dev = dev_nr;
#ifdef PCI_DEBUG
	printk(" - device: %x\n", dev_nr);
#endif	
				return (0);
			}
			indx++;
		}
	}
#ifdef PCI_DEBUG
	printk(" - not found\n");
#endif	
	return PCIBIOS_DEVICE_NOT_FOUND;
}    

const char *pcibios_strerror(int error)
{
	static char buf[32];
	switch (error)
	{	case PCIBIOS_SUCCESSFUL:
			return ("PCI BIOS: no error");
		case PCIBIOS_FUNC_NOT_SUPPORTED:
			return ("PCI BIOS: function not supported");
		case PCIBIOS_BAD_VENDOR_ID:
			return ("PCI BIOS: bad vendor ID");
		case PCIBIOS_DEVICE_NOT_FOUND:
			return ("PCI BIOS: device not found");
		case PCIBIOS_BAD_REGISTER_NUMBER:
			return ("PCI BIOS: bad register number");
		case PCIBIOS_SET_FAILED:
			return ("PCI BIOS: set failed");
		case PCIBIOS_BUFFER_TOO_SMALL:
			return ("PCI BIOS: buffer too small");
		default:
			sprintf(buf, "PCI BIOS: invalid error #%d", error);
			return(buf);
	}
}

// set up the PCI devices
void setup_nw_pci(void)
{
    volatile unsigned int * pci_device;

    *(volatile unsigned int* ) (IO_BASE_ARM_CSR + PCI_ADDR_EXT) = 0;        //Setting footbridge IO ext.to 0
    *(volatile unsigned char*) (IO_BASE_ARM_CSR + 0x0d) = 32;   // turn on bursting

    // set up ETHER10 address IO address 0x300
//    pci_device = (unsigned int*)0xE0900000;

    *(volatile int*) 0xE0900010 = 0x301;
    *(volatile char*)0xE0900004 = 1;	//Ether 10
    *(volatile char*)0xe090000d = 32;

    *(volatile int*) 0xE0180010 = 0x08000000;//intialize VGA PCI mem map
    *(volatile char*)0xE0180004 = 3;	//VGA
    *(volatile char*)0xE018000d = 32;

    /*
     * these slots are changable by the daughter card
     */
    pci_device= (unsigned int*) 0xE0300000;

    if (*pci_device==0x000D1000)
    {
        putstring("\npci: Symbios SCSI / fast ethernet detected\n");
        *(volatile int*)  0xE0300014 = 0x20010000;/* 64k window for registers */
        *(volatile int*)  0xE0300018 = 0x20011000;/* 64k window for scripts   */
        *(volatile short*)0xE030003C = 0x19;      /* W/H IRQ is 25 */
        *(volatile char*) 0xE030000c = 32;        /* cache line size  */
        *(volatile char*) 0xE030000d = 32;        /* bursting on  */
        *(volatile char*) 0xE0300004 = 0x6;       /* Bus Master + mem mapped  */
        *(volatile int*)  0xe0300110 = 0x500;
        *(volatile short*)0xE030013C = 0x19;      /* W/H IRQ is 25 SHARED!! */
        *(volatile char*) 0xE030010d = 32;        /* bursting on  */
        *(volatile char*) 0xE030010c = 32;        /* cache line size  */
        *(volatile char*) 0xE0300104 = 0x5;       /* Bus Master + io mapped */
    }
    else
    {
	pci_device= (unsigned int*) 0xE0200000;
	if ((short)*pci_device == PCI_VENDOR_ID_DEC)
	{
		putstring("\npci: DEC Tulip fast ethernet\n");
		// Tulip Memory Address : 0x20000000 (Phy Addr: 0xA0000000)
		*(volatile long*) 0xE0200014 = 0x20000000;
		*(volatile short*)0xE020003C = 0x19;	//W/H IRQ is 25
		*(volatile char *)0xE020000C = 0x20;	//PCI_CACHE_LINE_SIZE
		*(volatile char *)0xE020000D = 0x20;	//PCI_LATENCY_TIMER
		*(volatile char *)0xE0200004 = 0x6;     /* Bus Master + mem mapped  */
	}

	/* check if second DEC installed as well */	
	pci_device= (unsigned int*) 0xE0300000;
	if ((short)*pci_device == PCI_VENDOR_ID_DEC)
	{
		putstring("\npci: DEC Tulip fast ethernet II\n");
		// Tulip Memory Address : 0x20000100 (Phy Addr: 0xA0000100)
		*(volatile long*) 0xE0300014 = 0x20000100;
		*(volatile short*)0xE030003C = 0x19;	//W/H IRQ is 25
		*(volatile char *)0xE030000C = 0x20;	//PCI_CACHE_LINE_SIZE
		*(volatile char *)0xE030000D = 0x20;	//PCI_LATENCY_TIMER
		*(volatile char *)0xE0300004 = 0x6;     /* Bus Master + mem mapped  */
	}
    }

    //	*(volatile char*)0xE0300004 = 1;	//codec (unused for now)

    *(volatile char*)0xE050010D = 0x20;	//IDE PCI_LATENCY_TIMER
    *(volatile char*)0xE0500104 = 1;        //enable IDE controller as IO
}
