/*
 * linux/include/asm-arm/arch-vnc/io.h
 *
 * Copyright 1998 Corel Computer
 *
 * 981114 created
 */

#ifndef __ASM_ARM_ARCH_IO_H
#define __ASM_ARM_ARCH_IO_H


/* this architecture does not need io delay */
#undef ARCH_IO_DELAY

/*
 * IO -> physical
 */

extern __inline__ void __outb(unsigned int value, unsigned int port)
{
	volatile int i;
	i=0; i=0; // delay
    *(volatile char*)(IO_BASE + port) = value;
}

extern __inline__ void __outw(unsigned int value, unsigned int port)
{
    __asm__ (" strh %0,[%1,%2]" : : "r"(value), "r"(port), "r"(IO_BASE));
}

extern __inline__ void __outl(unsigned int value, unsigned int port)
{
    *(volatile int*)(IO_BASE + port) = value;
}




extern __inline__ unsigned char __inb(unsigned int port)
{
    return *(volatile char*)(IO_BASE + port);
}

extern __inline__ unsigned short __inw(unsigned int port)
{
    unsigned short s;

    __asm__("ldrh %0,[%1,%2]" : "=&r"(s) : "r"(port), "r"(IO_BASE));
    return s;
}

extern __inline__ unsigned long __inl(unsigned int port)
{
    return *(volatile unsigned long*)(port + IO_BASE);
}



extern __inline__ unsigned long __ioaddr(unsigned int port)
{
    return (unsigned long)(IO_BASE + port);
}


extern __inline__ void insl(unsigned int port, void *to, int len)
{
    unsigned long *p = (unsigned long*) to;
    unsigned long *e = p + len;
    /* optional...... */
    if ((int)to & 3)
        printk("ASSERT: insl misaligned buffer!!!\n");
    while (p<e)
        *p++ = *(volatile unsigned long*)(port+IO_BASE);
}

extern __inline__ void insw(unsigned int port, void *to, int len)
{
    unsigned short *p = (unsigned short*) to;
    unsigned short *e = p + len;
    while (p<e)
        __asm__ __volatile__ ("ldrh %0,[%1,%2]" : "=&r"(*p++) : "r"(port), "r"(IO_BASE));
        /* or, we could use this, as long as the compiler produces ldrh */
        // *p++ = *(unsigned volatile short*) (port + IO_BASE);
}

extern __inline__ void insb(unsigned int port, void *to, int len)
{
    unsigned char *p = (unsigned char*) to;
    unsigned char *e = p + len;
    while (p<e)
        *p++ = *(volatile unsigned char*)(port + IO_BASE);
}
extern __inline__ void outsl(unsigned int port, const void *from, int len)
{
    unsigned long *p = (unsigned long *)from;
    unsigned long *e = p + len;
    if ((int)from & 3)
        printk("ASSERT: insl misaligned buffer!!!\n");
    while(p<e)
        *(volatile unsigned long *)(port + IO_BASE) = *p++;
}
extern __inline__ void outsw(unsigned int port, const void *from, int len)
{
    unsigned short *p = (unsigned short *)from;
    unsigned short *e = p + len;
    unsigned short t;
    while(p<e) {
        t = *p++;
        __asm__ __volatile__ ("strh %0,[%1,%2]" : : "r"(t), "r"(port), "r"(IO_BASE));
        /* if we know the compiler produces strh, we can use this */
        /* __outw(*p++, port), len--; */
    }
}
extern __inline__ void outsb(unsigned int port, const void *from, int len)
{
    unsigned char *p = (unsigned char*) from;
    unsigned char *e = p + len;
    while (p<e)
        *(volatile unsigned char*)(port + IO_BASE) = *p++;
}



#endif /* __ASM_ARM_ARCH_IO_H */