#ifndef __ARM_UNALIGNED_H
#define __ARM_UNALIGNED_H

#include <asm/segment.h>

#define get_unaligned(ptr) \
	((__typeof__(*(ptr)))__get_unaligned((ptr), sizeof(*(ptr))))

#define put_unaligned(val, ptr) \
	__put_unaligned((unsigned long)(val), (ptr), sizeof(*(ptr)))

extern void bad_unaligned_access_length (void);
	
extern inline unsigned long __get_unaligned(const void *ptr, size_t size)
{
    unsigned long val;
    const unsigned char* p = (const unsigned char*)ptr;
    switch (size) {
    case 1:
        val = p[0];
        break;

    case 2:
        val = p[0] | (p[1]<<8);
        break;

    case 4:
        val = p[0] | (p[1]<<8) | (p[2]<<16) | (p[3]<<24);
        break;

    default:
        bad_unaligned_access_length ();
    }
    return val;
}

extern inline void __put_unaligned(unsigned long val, void *ptr, size_t size)
{
    unsigned char* p = (unsigned char*)ptr;
    switch (size) {
    case 1:
        p[0] = val;
        break;

    case 2:
        p[0] = val;
        p[1] = val>>8;
        break;

    case 4:
        p[0] = val;
        p[1] = val >> 8;
        p[2] = val >> 16;
        p[3] = val >> 24;
        break;

    default:
        bad_unaligned_access_length ();
    }
}

#endif
