/*
 *		IP_PORTFW_LBM_RR round robin load balancing  module
 *
 *
 * Version:	@(#)ip_portfw_lbm_rr.c 0.01   02/05/96
 *
 * Author:	Matthew Kellett <matthewk@corelcomputer.com>
 *		
 *  Converted from ip_masq_ftp.c
 *
 * Fixes:
 *	
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	
 */

#include <linux/module.h>
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/skbuff.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <net/protocol.h>
#include <net/tcp.h>
#include <net/ip_masq.h>

#ifndef DEBUG_CONFIG_IPPORTFW_LBM_RR 
#define DEBUG_CONFIG_IPPORTFW_LBM_RR 0
#endif

/* 
 * List of ports (up to MAX_MASQ_APP_PORTS) to be handled by helper
 * First port is set to the default port.
 */
struct ip_portfw_lbm *ip_portfw_lbm;

static int
ip_portfw_lbm_rr_init_1 (void **lbm_data)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif /*MODULE*/
	*lbm_data = NULL;
	return 0;
}

static int
ip_portfw_lbm_rr_done_1 (void **lbm_data)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif /*MODULE*/
	return 0;
}

struct server *
ip_portfw_lbm_rr_lookup (struct server *srv_lst, void **lbm_data)
{
	(struct server *)*lbm_data = ((struct server *)*lbm_data)->next;
	if (!*lbm_data)
	{
		(struct server *)*lbm_data = srv_lst;
	}
	return ((struct server *)*lbm_data);
}

int
ip_portfw_lbm_rr_server_change (struct server *srv_lst, void **lbm_data)
{
	(struct server *)*lbm_data = srv_lst;
	return 0;
}


struct ip_portfw_lbm ip_portfw_lbm_rr = {
	NULL,			/* next */
	"ip_portfw_lbm_rr",			/* name */
	0,                      /* n_attach */
	ip_portfw_lbm_rr_init_1,        /* ip_masq_init_1 */
	ip_portfw_lbm_rr_done_1,        /* ip_masq_done_1 */
	ip_portfw_lbm_rr_lookup,           /* pkt_out */
	ip_portfw_lbm_rr_server_change,    /* server addition or deletion */
};

/*
 * 	 initialization
 */

int ip_portfw_lbm_rr_init()
{
	int ret;
	if ((ip_portfw_lbm = kmalloc(sizeof(struct ip_portfw_lbm), GFP_KERNEL)) == NULL)
		return -ENOMEM;
	memcpy(ip_portfw_lbm, &ip_portfw_lbm_rr, sizeof(struct ip_portfw_lbm));
	if ((ret = register_ip_portfw_lbm(ip_portfw_lbm))) {
		printk(KERN_ERR "ip_portfw_lbm_rr: Unable to register module\n");
		return ret;
	}
	printk(KERN_DEBUG "ip_portfw_lbm_rr_init: Registered\n");
	return 0;
}

/*
 * 	ip_portfw_lbm_rr fin.
 */

int ip_portfw_lbm_rr_done()
{
	int ret;
	if ((ret = unregister_ip_portfw_lbm(ip_portfw_lbm))) 
		return ret;
	kfree(ip_portfw_lbm);
	printk(KERN_DEBUG "ip_portfw_lbm_rr_done: Unregistered\n");
	return 0;
}

#ifdef MODULE

int init_module(void)
{
	if (ip_portfw_lbm_rr_init() != 0)
		return -EIO;
	register_symtab(0);
	return 0;
}

void cleanup_module(void)
{
	if (ip_portfw_lbm_rr_done() != 0)
		printk(KERN_ERR "ip_portfw_lbm_rr: can't remove module\n");
}

#endif /* MODULE */
