/*
 *		IP_PORTFW_LBM_ZAPHOD weighted round robin load balancing  module
 *
 *
 * Version:	@(#)ip_portfw_lbm_zaphod.c 0.01   02/05/96
 *
 * Author:	Matthew Kellett <matthewk@corelcomputer.com>
 *		
 *  Converted from ip_masq_ftp.c
 *
 * Fixes:
 *	
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	
 */

#include <linux/module.h>
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/skbuff.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/time.h>
#include <net/protocol.h>
#include <net/tcp.h>
#include <net/ip_masq.h>

#ifndef DEBUG_CONFIG_IPPORTFW_LBM_ZAPHOD 
#define DEBUG_CONFIG_IPPORTFW_LBM_ZAPHOD 0
#endif

#define NITER 4

struct ip_portfw_lbm *ip_portfw_lbm;

static int ran_counter=1;

void ip_portfw_lbm_zaphod_psdes (unsigned long *lword, unsigned long *irword);
unsigned long ip_portfw_lbm_zaphod_random(long idum);

void ip_portfw_lbm_zaphod_psdes (unsigned long *lword, unsigned long *irword)
{
	unsigned long i, ia, ib, iswap, itmph=0, itmpl=0;
	static unsigned long c1[NITER]={
			0xbaa96887L, 0x1e17d32cL, 0x03bcdc3cL, 0x0f33d1b2L};
	static unsigned long c2[NITER]={
			0x4b0f3b58L, 0xe874f0c3L, 0x6955c5a6L, 0x55a7ca46L};

	for (i=1; i<NITER; i++) {
		ia=(iswap=(*irword)) ^ c1[i];
		itmpl = ia & 0xffff;
		itmph = ia >> 16;
		ib=itmpl*itmpl+ ~(itmph*itmph);
		*irword=(*lword) ^ (((ia = (ib>> 16) |
			((ib & 0xffff) << 16)) ^ c2[i])+itmpl*itmph);
		*lword=iswap;
	}
}

unsigned long ip_portfw_lbm_zaphod_random(long idum)
{
	unsigned long irword, lword;
	static long idums=0;
	if (idum < 0) {
		idums = -idum;
		idum=1;
	}
	irword=idum;
	lword=idums;
	ip_portfw_lbm_zaphod_psdes(&lword, &irword);
	return irword;
}

static int
ip_portfw_lbm_zaphod_init_1 (void **lbm_data)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif /*MODULE*/
	*lbm_data = NULL;
	return 0;
}

static int
ip_portfw_lbm_zaphod_done_1 (void **lbm_data)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif /*MODULE*/
	return 0;
}

struct server *
ip_portfw_lbm_zaphod_lookup (struct server *srv_lst, void **lbm_data)
{
	unsigned long ran_number=0, max=0;
	struct server *srv=NULL, *chosen=NULL;
	
	do {
		ran_number=ip_portfw_lbm_zaphod_random(++ran_counter);
		srv = srv_lst;
		chosen = NULL;
		while ((!chosen) && (srv)) {
			if (srv->weight >= ran_number)
				chosen = srv;
			srv = srv->next;
		}
		max++;
	} while ((chosen == *lbm_data) && (max < 5));
	*lbm_data = chosen;
	printk(KERN_DEBUG "ip_portfw_lbm_zaphod_lookup: Chose %p(%08x) weight=%08lx ran=%08lx max=%08lx\n", chosen, chosen->ip, chosen->weight, ran_number, max);

	return chosen;
}

int
ip_portfw_lbm_zaphod_server_change (struct server *srv_lst, void **lbm_data)
{
	struct timeval tv;
	long seed, count=0, total=0;
	struct server *srv;

	do_gettimeofday(&tv);
	seed = -(tv.tv_sec);
	ip_portfw_lbm_zaphod_random(seed);

	for (srv=srv_lst; srv; srv=srv->next) {
		if ((srv->memory) && (srv->load)) {
			total += ((srv->memory * 1024)/srv->load);
		}
		else {
			total += 1;
		}
	}

	for (srv=srv_lst; srv; srv=srv->next) {
		if ((srv->memory) && (srv->load)) 
			count += ((srv->memory * 1024)/srv->load);
		else 
			count += 1;
		
		if (count != total) 
			srv->weight = count * (0xffffffff/total);
		else
			srv->weight = 0xffffffff;
	}

	*lbm_data = srv_lst;
	if (ran_counter > 0xffff)
		ran_counter = 0;
	return 0;
}

struct ip_portfw_lbm ip_portfw_lbm_zaphod = {
	NULL,			/* next */
	"ip_portfw_lbm_zaphod",			/* name */
	0,                      /* n_attach */
	ip_portfw_lbm_zaphod_init_1,        /* ip_masq_init_1 */
	ip_portfw_lbm_zaphod_done_1,        /* ip_masq_done_1 */
	ip_portfw_lbm_zaphod_lookup,           /* pkt_out */
	ip_portfw_lbm_zaphod_server_change,    /* server addition or deletion */
};

/*
 * 	 initialization
 */

int ip_portfw_lbm_zaphod_init()
{
	int ret;
	if ((ip_portfw_lbm = kmalloc(sizeof(struct ip_portfw_lbm), GFP_KERNEL)) == NULL)
		return -ENOMEM;
	memcpy(ip_portfw_lbm, &ip_portfw_lbm_zaphod, sizeof(struct ip_portfw_lbm));
	if ((ret = register_ip_portfw_lbm(ip_portfw_lbm))) {
		printk(KERN_ERR "ip_portfw_lbm_zaphod: Unable to register module\n");
		return ret;
	}
	printk(KERN_DEBUG "ip_portfw_lbm_zaphod_init: Registered\n");
	return 0;
}

/*
 * 	ip_portfw_lbm_zaphod fin.
 */

int ip_portfw_lbm_zaphod_done()
{
	int ret;
	if ((ret = unregister_ip_portfw_lbm(ip_portfw_lbm))) 
		return ret;
	kfree(ip_portfw_lbm);
	printk(KERN_DEBUG "ip_portfw_lbm_zaphod_done: Unregistered\n");
	return 0;
}

#ifdef MODULE

int init_module(void)
{
	if (ip_portfw_lbm_zaphod_init() != 0)
		return -EIO;
	register_symtab(0);
	return 0;
}

void cleanup_module(void)
{
	if (ip_portfw_lbm_zaphod_done() != 0)
		printk(KERN_ERR "ip_portfw_lbm_zaphod: can't remove module\n");
}

#endif /* MODULE */
